package it.tref.liferay.mapit.map.util;

import it.tref.liferay.mapit.map.constants.PortletKeys;
import it.tref.liferay.mapit.map.hook.deploy.DeployUtil;
import it.tref.liferay.mapit.map.model.Plugin;
import it.tref.liferay.mapit.map.service.PluginLocalServiceUtil;
import it.tref.liferay.mapit.plugin.MapItPlugin;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Proxy;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.ConcurrentHashMap;

import com.liferay.portal.kernel.dao.orm.QueryUtil;
import com.liferay.portal.kernel.deploy.DeployManagerUtil;
import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.plugin.PluginPackage;
import com.liferay.portal.kernel.plugin.Version;
import com.liferay.portal.kernel.util.ClassLoaderPool;
import com.liferay.portal.kernel.util.ClassResolverUtil;
import com.liferay.portal.kernel.util.PropertiesUtil;
import com.liferay.portal.kernel.util.StreamUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.Validator;

public class MapItPluginUtil {

  private final static String PROPERTIES = "portlet.properties";
  private final static String KEY_MAPIT_PLUGIN = "mapit.plugin";
  private final static String KEY_MAPIT_VERSION = "mapit.version";

  private final static Log _log = LogFactoryUtil.getLog(MapItPluginUtil.class);

  private final static Map<String, MapItPlugin> _mapItPlugins = new ConcurrentHashMap<String, MapItPlugin>();

  public static MapItPlugin getMapItPlugin(String contextName) {
    return _mapItPlugins.get(contextName);
  }

  public static boolean contains(String contextName) {
    return _mapItPlugins.containsKey(contextName);
  }

  public static void register(String contextName, ClassLoader contextClassLoader, String mapItPluginClass)
      throws Exception {

    Class<?> clazz = ClassResolverUtil.resolve(mapItPluginClass, contextClassLoader);
    Object obj = clazz.getConstructor(new Class[0]).newInstance(new Object[0]);

    ClassLoader currentClassLoader = MapItPlugin.class.getClassLoader();

    MapItPlugin mapItPlugin =
        (MapItPlugin) Proxy.newProxyInstance(currentClassLoader, new Class[] {MapItPlugin.class},
            new MapItPluginInvocationHandler(obj));

    _mapItPlugins.put(contextName, mapItPlugin);
  }

  public static void unregister(String contextName) {
    _mapItPlugins.remove(contextName);
  }

  public static void searchAndUpdatePlugins() throws SystemException {

    // delete plugins
    List<Plugin> plugins = PluginLocalServiceUtil.getPlugins(QueryUtil.ALL_POS, QueryUtil.ALL_POS);
    for (Plugin plugin : plugins) {
      PluginPackage pluginPackage = DeployManagerUtil.getInstalledPluginPackage(plugin.getContext());
      if (pluginPackage == null) {
        PluginLocalServiceUtil.deletePlugin(plugin);
      }
    }

    // add or update plugins
    List<PluginPackage> pluginsPackage = DeployManagerUtil.getInstalledPluginPackages();
    for (PluginPackage pluginPackage : pluginsPackage) {
      ClassLoader contextClassLoader = ClassLoaderPool.getClassLoader(pluginPackage.getContext());
      DeployUtil.checkMapItPluginOnDeploy(pluginPackage, contextClassLoader);
    }
  }

  public static String getMapItPluginClass(ClassLoader contextClassLoader) {

    String mapItPluginClass = null;

    InputStream inputStream = null;

    try {

      inputStream = contextClassLoader.getResourceAsStream(PROPERTIES);

      if (inputStream != null) {

        try {

          Properties properties = PropertiesUtil.load(inputStream, StringPool.UTF8);

          mapItPluginClass = properties.getProperty(KEY_MAPIT_PLUGIN);

        } catch (IOException e) {
          _log.warn("Properties file", e);
        }
      }
    } finally {
      StreamUtil.cleanUp(inputStream);
    }

    return mapItPluginClass;
  }

  public static boolean isMapItPlugin(String mapItPluginClass, ClassLoader contextClassLoader) {

    if (Validator.isNotNull(mapItPluginClass)) {

      try {
        ClassResolverUtil.resolve(mapItPluginClass, contextClassLoader);
        return true;
      } catch (Exception e) {
      }
    }

    return false;
  }

  public static boolean isMapItCompatiblePlugin(String mapItPluginVersion) {

    if (Validator.isNull(mapItPluginVersion)) {
      return false;
    }

    String currentMapitVersion = getMapItVersion();

    return Version.getInstance(mapItPluginVersion).includes(Version.getInstance(currentMapitVersion));
  }

  public static boolean isMapItCompatiblePlugin(long pluginId) throws SystemException, PortalException {
    return isMapItCompatiblePlugin(PluginLocalServiceUtil.getPlugin(pluginId));
  }

  public static String getMapItPluginVersion(ClassLoader contextClassLoader) {

    String mapItPluginVersion = null;

    InputStream inputStream = null;

    try {

      inputStream = contextClassLoader.getResourceAsStream(PROPERTIES);

      if (inputStream != null) {

        try {

          Properties properties = PropertiesUtil.load(inputStream, StringPool.UTF8);

          mapItPluginVersion = properties.getProperty(KEY_MAPIT_VERSION);

        } catch (IOException e) {
          _log.warn("Properties file", e);
        }
      }
    } finally {
      StreamUtil.cleanUp(inputStream);
    }

    return mapItPluginVersion;
  }

  public static boolean isMapItCompatiblePlugin(Plugin plugin) throws SystemException {

    String mapItPluginVersion = plugin.getMapItVersion();

    if (Validator.isNull(mapItPluginVersion)) {
      return false;
    }

    String currentMapitVersion = getMapItVersion();

    return Version.getInstance(mapItPluginVersion).includes(Version.getInstance(currentMapitVersion));
  }

  private static String getMapItVersion() {

    try {
      PluginPackage pluginPackage = DeployManagerUtil.getInstalledPluginPackage(PortletKeys.MAP_IT_MAP_CONTEXT_NAME);
      return pluginPackage.getVersion();
    } catch (Exception e) {
      return null;
    }
  }

}
